import {
  Component,
  OnInit,
  OnDestroy,
  AfterContentInit,
  ChangeDetectorRef
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { Observable, Subscription } from 'rxjs';
import { map, catchError, take, tap } from 'rxjs/operators';

import { Port, Service, Host } from '@overflow/model/discovery';

import { TargetDisplayType, DiscoveryStatusType } from '../../../core/type';
import * as AppStore from '../../../store';
import * as TargetStore from '../../../store/target/target';


@Component({
  selector: 'app-target-display',
  templateUrl: './display.component.html',
  styleUrls: ['./display.component.scss']
})
export class DisplayComponent implements OnInit, AfterContentInit, OnDestroy {
  private targetDisplayType$: Observable<TargetDisplayType>;

  displaySidebar = false;
  selectedTargetSubscription: Subscription | null;
  selectedTarget: { group: string, target: Host | Port | Service } | null;

  displayDiscoveryStopping: boolean;
  discoveryStatusSubscription: Subscription | null;

  constructor(
    private store: Store<any>,
    private changeDetector: ChangeDetectorRef,
  ) {
  }

  ngOnInit(): void {
    const __this = this;

    this.targetDisplayType$ = this.store.pipe(select(AppStore.TargetSelector.TargetSelector.selectTargetDisplayType));

    this.selectedTargetSubscription = this.store.pipe(
      select(AppStore.TargetSelector.TargetSelector.selectSelectedTarget)
    ).pipe(
      map((_selectedTarget) => {
        if (null === _selectedTarget) {
          __this.displaySidebar = false;
          __this.selectedTarget = null;
        } else {
          __this.displaySidebar = true;
          __this.selectedTarget = _selectedTarget;
        }
        __this.changeDetector.detectChanges();
      }),
    ).subscribe();

    this.discoveryStatusSubscription = this.store.pipe(
      select(AppStore.DiscoverySelector.RequestSelector.selectDiscoveryStatus)
    ).pipe(
      map((_discoveryStatus) => {
        switch (_discoveryStatus) {
          case DiscoveryStatusType.Stopping: {
            __this.displayDiscoveryStopping = true;
          }
            break;
          default: {
            __this.displayDiscoveryStopping = false;

          }
            break;
        }
        __this.changeDetector.detectChanges();
      }),
    ).subscribe();

  }

  ngAfterContentInit(): void {
  }

  ngOnDestroy(): void {
    if (null !== this.selectedTargetSubscription) {
      this.selectedTargetSubscription.unsubscribe();
    }
    if (null !== this.discoveryStatusSubscription) {
      this.discoveryStatusSubscription.unsubscribe();
    }
  }

  onHideDetail() {
    this.store.dispatch(new TargetStore.ChangeSelectedTarget(null));
  }

}
